MODULE asmphytobal_medusa
   !!======================================================================
   !!                       ***  MODULE asmphyto2dbal_medusa  ***
   !! Calculate increments to MEDUSA based on surface phyto2d increments
   !!
   !! IMPORTANT NOTE: This calls the bioanalysis routine of Hemmings et al.
   !! For licensing reasons this is kept in its own internal Met Office
   !! branch (dev/frdf/vn3.6_nitrogen_balancing) rather than in the Paris
   !! repository, and must be merged in when building.
   !!
   !!======================================================================
   !! History :  3.6  ! 2017-08 (D. Ford)  Adapted from asmphyto2dbal_hadocc
   !!----------------------------------------------------------------------
#if defined key_asminc && defined key_medusa
   !!----------------------------------------------------------------------
   !! 'key_asminc'          : assimilation increment interface
   !! 'key_medusa'          : MEDUSA model
   !!----------------------------------------------------------------------
   !! asm_phyto2d_bal_medusa : routine to calculate increments to MEDUSA
   !!----------------------------------------------------------------------
   USE par_kind,      ONLY: wp             ! kind parameters
   USE par_oce,       ONLY: jpi, jpj, jpk  ! domain array sizes
   USE dom_oce,       ONLY: gdepw_n        ! domain information
   USE zdftmx,        ONLY: ln_tmx_itf, &  ! Indonesian Throughflow
      &                     mask_itf       ! tidal mixing mask
   USE iom                                 ! i/o
   USE sms_medusa                          ! MEDUSA parameters
   USE par_medusa                          ! MEDUSA parameters
   USE par_trc,       ONLY: jptra          ! Tracer parameters
   USE bioanalysis                         ! Nitrogen balancing

   IMPLICIT NONE
   PRIVATE                   

   PUBLIC asm_phyto_bal_medusa

   ! Default values for biological assimilation parameters
   ! Should match Hemmings et al. (2008)
   REAL(wp), PARAMETER :: balnutext  =  0.6    !: Default nutrient balancing factor
   REAL(wp), PARAMETER :: balnutmin  =  0.1    !: Fraction of phytoplankton loss to nutrient
   REAL(wp), PARAMETER :: r          =  1      !: Reliability of model specific growth rate
   REAL(wp), PARAMETER :: beta_g     =  0.05   !: Low rate bias correction for growth rate estimator
   REAL(wp), PARAMETER :: beta_l     =  0.05   !: Low rate bias correction for primary loss rate estimator
   REAL(wp), PARAMETER :: beta_m     =  0.05   !: Low rate bias correction for secondary loss rate estimator
   REAL(wp), PARAMETER :: a_g        =  0.2    !: Error s.d. for log10 of growth rate estimator
   REAL(wp), PARAMETER :: a_l        =  0.4    !: Error s.d. for log10 of primary loss rate estimator
   REAL(wp), PARAMETER :: a_m        =  0.7    !: Error s.d. for log10 of secondary loss rate estimator
   REAL(wp), PARAMETER :: zfracb0    =  0.7    !: Base zooplankton fraction of loss to Z & D
   REAL(wp), PARAMETER :: zfracb1    =  0      !: Phytoplankton sensitivity of zooplankton fraction
   REAL(wp), PARAMETER :: qrfmax     =  1.1    !: Maximum nutrient limitation reduction factor
   REAL(wp), PARAMETER :: qafmax     =  1.1    !: Maximum nutrient limitation amplification factor
   REAL(wp), PARAMETER :: zrfmax     =  2      !: Maximum zooplankton reduction factor
   REAL(wp), PARAMETER :: zafmax     =  2      !: Maximum zooplankton amplification factor
   REAL(wp), PARAMETER :: prfmax     =  10     !: Maximum phytoplankton reduction factor (secondary)
   REAL(wp), PARAMETER :: incphymin  =  0.0001 !: Minimum size of non-zero phytoplankton increment
   REAL(wp), PARAMETER :: integnstep =  20     !: Number of steps for p.d.f. integral evaluation
   REAL(wp), PARAMETER :: pthreshold =  0.01   !: Fractional threshold level for setting p.d.f.
   !
   LOGICAL,  PARAMETER :: diag_active           = .TRUE.  !: Depth-independent diagnostics
   LOGICAL,  PARAMETER :: diag_fulldepth_active = .TRUE.  !: Full-depth diagnostics
   LOGICAL,  PARAMETER :: gl_active             = .TRUE.  !: Growth/loss-based balancing
   LOGICAL,  PARAMETER :: nbal_active           = .TRUE.  !: Nitrogen balancing
   LOGICAL,  PARAMETER :: subsurf_active        = .TRUE.  !: Increments below MLD
   LOGICAL,  PARAMETER :: deepneg_active        = .FALSE. !: Negative primary increments below MLD
   LOGICAL,  PARAMETER :: deeppos_active        = .FALSE. !: Positive primary increments below MLD
   LOGICAL,  PARAMETER :: nutprof_active        = .TRUE.  !: Secondary increments

CONTAINS

   SUBROUTINE asm_phyto_bal_medusa( kdeps,                              &
      &                             ld_chltot,                          &
      &                             pinc_chltot_3d,                     &
      &                             ld_chldia,                          &
      &                             pinc_chldia_3d,                     &
      &                             ld_chlnon,                          &
      &                             pinc_chlnon_3d,                     &
      &                             ld_phytot,                          &
      &                             pinc_phytot_3d,                     &
      &                             ld_phydia,                          &
      &                             pinc_phydia_3d,                     &
      &                             ld_phynon,                          &
      &                             pinc_phynon_3d,                     &
      &                             pincper,                            &
      &                             p_maxchlinc, ld_phytobal, pmld,     &
      &                             pgrow_avg_bkg_3d, ploss_avg_bkg_3d, &
      &                             phyt_avg_bkg_3d, mld_max_bkg,       &
      &                             tracer_bkg, phyto_balinc )
      !!---------------------------------------------------------------------------
      !!                    ***  ROUTINE asm_phyto_bal_medusa  ***
      !!
      !! ** Purpose :   calculate increments to MEDUSA from phytoplankton increments
      !!
      !! ** Method  :   average up MEDUSA to look like HadOCC
      !!                call nitrogen balancing scheme
      !!                separate back out to MEDUSA
      !!
      !! ** Action  :   populate phyto_balinc
      !!
      !! References :   Hemmings et al., 2008, J. Mar. Res.
      !!                Ford et al., 2012, Ocean Sci.
      !!---------------------------------------------------------------------------
      !!
      INTEGER,  INTENT(in   )                               :: kdeps            ! No. inc deps 1 or jpk
      LOGICAL,  INTENT(in   )                               :: ld_chltot        ! Assim chltot y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_chltot_3d   ! chltot increments (3D)
      LOGICAL,  INTENT(in   )                               :: ld_chldia        ! Assim chldia y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_chldia_3d   ! chldia increments (3D)
      LOGICAL,  INTENT(in   )                               :: ld_chlnon        ! Assim chlnon y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_chlnon_3d   ! chlnon increments (3D)
      LOGICAL,  INTENT(in   )                               :: ld_phytot        ! Assim phytot y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_phytot_3d   ! phytot increments (3D)
      LOGICAL,  INTENT(in   )                               :: ld_phydia        ! Assim phydia y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_phydia_3d   ! phydia increments (3D)
      LOGICAL,  INTENT(in   )                               :: ld_phynon        ! Assim phynon y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_phynon_3d   ! phynon increments (3D)
      REAL(wp), INTENT(in   )                               :: pincper          ! Assimilation period
      REAL(wp), INTENT(in   )                               :: p_maxchlinc      ! Max chl increment
      LOGICAL,  INTENT(in   )                               :: ld_phytobal      ! Balancing y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pmld             ! Mixed layer depth
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,kdeps)     :: pgrow_avg_bkg_3d ! Avg phyto growth (3D)
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,kdeps)     :: ploss_avg_bkg_3d ! Avg phyto loss (3D)
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,kdeps)     :: phyt_avg_bkg_3d  ! Avg phyto (3D)
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj)           :: mld_max_bkg      ! Max MLD
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk,jptra) :: tracer_bkg       ! State variables
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk,jptra) :: phyto_balinc     ! Balancing increments
      !!
      INTEGER                                               :: ji, jj, jk, jn ! Loop counters
      INTEGER                                               :: jkmax          ! Loop index
      INTEGER                                               :: jkinc          ! Loop index
      INTEGER,                 DIMENSION(6)                 :: i_tracer       ! Tracer indices
      REAL(wp)                                              :: n2be_p         ! N:biomass for total phy
      REAL(wp)                                              :: n2be_z         ! N:biomass for total zoo
      REAL(wp)                                              :: n2be_d         ! N:biomass for detritus
      REAL(wp)                                              :: zfrac          ! Fraction
      REAL(wp)                                              :: zfrac_chn      ! Fraction of jpchn
      REAL(wp)                                              :: zfrac_chd      ! Fraction of jpchd
      REAL(wp)                                              :: zfrac_phn      ! Fraction of jpphn
      REAL(wp)                                              :: zfrac_phd      ! Fraction of jpphd
      REAL(wp)                                              :: zfrac_zmi      ! Fraction of jpzmi
      REAL(wp)                                              :: zfrac_zme      ! Fraction of jpzme
      REAL(wp)                                              :: zrat_pds_phd   ! Ratio of jppds:jpphd
      REAL(wp)                                              :: zrat_chd_phd   ! Ratio of jpchd:jpphd
      REAL(wp)                                              :: zrat_chn_phn   ! Ratio of jpchn:jpphn
      REAL(wp)                                              :: zrat_phn_chn   ! Ratio of jpphn:jpchn
      REAL(wp)                                              :: zrat_phd_chd   ! Ratio of jpphd:jpchd
      REAL(wp)                                              :: zrat_pds_chd   ! Ratio of jppds:jpchd
      REAL(wp)                                              :: zrat_dtc_det   ! Ratio of jpdtc:jpdet
      REAL(wp),                DIMENSION(jpi,jpj)           :: cchl_p_2d      ! C:Chl for total phy (2D)
      REAL(wp),                DIMENSION(jpi,jpj,jpk)       :: cchl_p_3d      ! C:Chl for total phy (3D)
      REAL(wp),                DIMENSION(16)                :: modparm        ! Model parameters
      REAL(wp),                DIMENSION(20)                :: assimparm      ! Assimilation parameters
      REAL(wp),                DIMENSION(jpi,jpj,1,6)       :: bstate_2d      ! Background state (2D)
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: bstate_3d      ! Background state (3D)
      REAL(wp),                DIMENSION(jpi,jpj,1,6)       :: outincs_2d     ! Balancing increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: outincs_3d     ! Balancing increments (3D)
      REAL(wp),                DIMENSION(jpi,jpj,22)        :: diag              ! Depth-indep diagnostics
      REAL(wp),                DIMENSION(jpi,jpj,1,22)      :: diag_fulldepth_2d ! Full-depth diagnostics (2D)
      REAL(wp),                DIMENSION(jpi,jpj,jpk,22)    :: diag_fulldepth_3d ! Full-depth diagnostics (3D)
      REAL(wp),                DIMENSION(jpi,jpj,1)         :: tmask_2d          ! Single-level tmask
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_chltot_2d    ! chltot increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_chldia_2d    ! chldia increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_chlnon_2d    ! chlnon increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_phytot_2d    ! phytot increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_phydia_2d    ! phydia increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_phynon_2d    ! phynon increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pgrow_avg_bkg_2d  ! Avg phyto growth (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: ploss_avg_bkg_2d  ! Avg phyto loss (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: phyt_avg_bkg_2d   ! Avg phyto (2D)
      !!---------------------------------------------------------------------------

      ! If p_maxchlinc > 0 then cap total absolute chlorophyll increment at that value
      IF ( p_maxchlinc > 0.0 ) THEN
         DO jk = 1, kdeps
            DO jj = 1, jpj
               DO ji = 1, jpi
                  IF ( ld_chltot ) THEN
                     pinc_chltot_3d(ji,jj,jk) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chltot_3d(ji,jj,jk), p_maxchlinc ) )
                  ELSE IF ( ld_chldia .AND. ld_chlnon ) THEN
                     pinc_chltot_3d(ji,jj,jk) = pinc_chldia_3d(ji,jj,jk) + pinc_chlnon_3d(ji,jj,jk)
                     pinc_chltot_3d(ji,jj,jk) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chltot_3d(ji,jj,jk), p_maxchlinc ) )
                     IF ( pinc_chltot_3d(ji,jj,jk) .NE. ( pinc_chldia_3d(ji,jj,jk) + pinc_chlnon_3d(ji,jj,jk) ) ) THEN
                        zfrac = pinc_chltot_3d(ji,jj,jk) / ( pinc_chldia_3d(ji,jj,jk) + pinc_chlnon_3d(ji,jj,jk) )
                        pinc_chldia_3d(ji,jj,jk) = pinc_chldia_3d(ji,jj,jk) * zfrac
                        pinc_chlnon_3d(ji,jj,jk) = pinc_chlnon_3d(ji,jj,jk) * zfrac
                     ENDIF
                  ELSE IF ( ld_chldia ) THEN
                     pinc_chldia_3d(ji,jj,jk) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chldia_3d(ji,jj,jk), p_maxchlinc ) )
                     pinc_chltot_3d(ji,jj,jk) = pinc_chldia_3d(ji,jj,jk)
                  ELSE IF ( ld_chlnon ) THEN
                     pinc_chlnon_3d(ji,jj,jk) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chlnon_3d(ji,jj,jk), p_maxchlinc ) )
                     pinc_chltot_3d(ji,jj,jk) = pinc_chlnon_3d(ji,jj,jk)
                  ENDIF
               END DO
            END DO
         END DO
      ENDIF

      IF ( ld_phytot .OR. ld_phydia .OR. ld_phynon ) THEN
         CALL ctl_stop( ' No phytoplankton carbon assimilation quite yet' )
      ENDIF
      
      IF ( ld_phytobal ) THEN   ! Nitrogen balancing

         ! Set up model parameters to be passed into Hemmings balancing routine.
         ! For now these are hardwired to the standard HadOCC parameter values
         ! (except C:N ratios) as this is what the scheme was developed for.
         ! Obviously, HadOCC and MEDUSA are rather different models, so this
         ! isn't ideal, but there's not always direct analogues between the two
         ! parameter sets, so it's the easiest way to get something running.
         ! In the longer term, some serious MarMOT-based development is required.
         modparm(1)  = 0.1                               ! grow_sat
         modparm(2)  = 2.0                               ! psmax
         modparm(3)  = 0.845                             ! par
         modparm(4)  = 0.02                              ! alpha
         modparm(5)  = 0.05                              ! resp_rate
         modparm(6)  = 0.05                              ! pmort_rate
         modparm(7)  = 0.01                              ! phyto_min
         modparm(8)  = 0.05                              ! z_mort_1
         modparm(9)  = 1.0                               ! z_mort_2
         modparm(10) = ( xthetapn  + xthetapd  ) / 2.0   ! c2n_p
         modparm(11) = ( xthetazmi + xthetazme ) / 2.0   ! c2n_z
         modparm(12) = xthetad                           ! c2n_d
         modparm(13) = 0.01                              ! graze_threshold
         modparm(14) = 2.0                               ! holling_coef
         modparm(15) = 0.5                               ! graze_sat
         modparm(16) = 2.0                               ! graze_max

         ! Set up assimilation parameters to be passed into balancing routine
         ! Not sure what assimparm(1) is meant to be, but it doesn't get used
         assimparm(2)  = balnutext
         assimparm(3)  = balnutmin
         assimparm(4)  = r
         assimparm(5)  = beta_g
         assimparm(6)  = beta_l
         assimparm(7)  = beta_m
         assimparm(8)  = a_g
         assimparm(9)  = a_l
         assimparm(10) = a_m
         assimparm(11) = zfracb0
         assimparm(12) = zfracb1
         assimparm(13) = qrfmax
         assimparm(14) = qafmax
         assimparm(15) = zrfmax
         assimparm(16) = zafmax
         assimparm(17) = prfmax
         assimparm(18) = incphymin
         assimparm(19) = integnstep
         assimparm(20) = pthreshold

         ! Set up external tracer indices array bstate
         i_tracer(1) = 1   ! nutrient
         i_tracer(2) = 2   ! phytoplankton
         i_tracer(3) = 3   ! zooplankton
         i_tracer(4) = 4   ! detritus
         i_tracer(5) = 5   ! DIC
         i_tracer(6) = 6   ! Alkalinity

         ! Set background state
         bstate_3d(:,:,:,i_tracer(1)) = tracer_bkg(:,:,:,jpdin)
         bstate_3d(:,:,:,i_tracer(2)) = tracer_bkg(:,:,:,jpphn) + tracer_bkg(:,:,:,jpphd)
         bstate_3d(:,:,:,i_tracer(3)) = tracer_bkg(:,:,:,jpzmi) + tracer_bkg(:,:,:,jpzme)
         bstate_3d(:,:,:,i_tracer(4)) = tracer_bkg(:,:,:,jpdet)
         bstate_3d(:,:,:,i_tracer(5)) = tracer_bkg(:,:,:,jpdic)
         bstate_3d(:,:,:,i_tracer(6)) = tracer_bkg(:,:,:,jpalk)

         ! Calculate carbon to chlorophyll ratio for combined phytoplankton
         ! and nitrogen to biomass equivalent for PZD
         ! Hardwire nitrogen mass to 14.01 for now as it doesn't seem to be set in MEDUSA
         cchl_p_3d(:,:,:) = 0.0
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  IF ( ( tracer_bkg(ji,jj,jk,jpchn) + tracer_bkg(ji,jj,jk,jpchd ) ) .GT. 0.0 ) THEN
                     cchl_p_3d(ji,jj,jk) = xmassc * ( ( tracer_bkg(ji,jj,jk,jpphn) * xthetapn ) +      &
                        &                             ( tracer_bkg(ji,jj,jk,jpphd) * xthetapd )   ) /  &
                        &                  ( tracer_bkg(ji,jj,jk,jpchn) + tracer_bkg(ji,jj,jk,jpchd ) )
                  ENDIF
               END DO
            END DO
         END DO
         n2be_p = 14.01 + ( xmassc * ( ( xthetapn  + xthetapd  ) / 2.0 ) )
         n2be_z = 14.01 + ( xmassc * ( ( xthetazmi + xthetazme ) / 2.0 ) )
         n2be_d = 14.01 + ( xmassc * xthetad )

         ! Call nitrogen balancing routine
         IF (kdeps == 1) THEN
            pinc_chltot_2d(:,:)   = pinc_chltot_3d(:,:,1)
            cchl_p_2d(:,:)        = cchl_p_3d(:,:,1)
            phyt_avg_bkg_2d(:,:)  = phyt_avg_bkg_3d(:,:,1)
            pgrow_avg_bkg_2d(:,:) = pgrow_avg_bkg_3d(:,:,1)
            ploss_avg_bkg_2d(:,:) = ploss_avg_bkg_3d(:,:,1)
            
            CALL bio_analysis( jpi, jpj, jpk, gdepw_n(:,:,2:jpk), i_tracer, modparm,   &
               &               n2be_p, n2be_z, n2be_d, assimparm,                      &
               &               INT(pincper), 1, INT(SUM(tmask,3)), tmask(:,:,:),       &
               &               pmld(:,:), mld_max_bkg(:,:), pinc_chltot_2d(:,:), cchl_p_2d(:,:), &
               &               nbal_active, phyt_avg_bkg_2d(:,:),                      &
               &               gl_active, pgrow_avg_bkg_2d(:,:), ploss_avg_bkg_2d(:,:), &
               &               subsurf_active, deepneg_active,                         &
               &               deeppos_active, nutprof_active,                         &
               &               bstate_3d, outincs_3d,                                  &
               &               diag_active, diag,                                      &
               &               diag_fulldepth_active, diag_fulldepth_3d )
         ELSE
            pmld(:,:) = 0.5
            
            DO jk = 1, kdeps
               pinc_chltot_2d(:,:)   = pinc_chltot_3d(:,:,jk)
               cchl_p_2d(:,:)        = cchl_p_3d(:,:,jk)
               phyt_avg_bkg_2d(:,:)  = phyt_avg_bkg_3d(:,:,jk)
               pgrow_avg_bkg_2d(:,:) = pgrow_avg_bkg_3d(:,:,jk)
               ploss_avg_bkg_2d(:,:) = ploss_avg_bkg_3d(:,:,jk)
               tmask_2d(:,:,1)       = tmask(:,:,jk)
               bstate_2d(:,:,1,:)    = bstate_3d(:,:,jk,:)
               outincs_2d(:,:,:,:)   = 0.0

               CALL bio_analysis( jpi, jpj, 1, gdepw_n(:,:,2), i_tracer, modparm,            &
                  &               n2be_p, n2be_z, n2be_d, assimparm,                         &
                  &               INT(pincper), 1, INT(SUM(tmask_2d,3)), tmask_2d(:,:,:),    &
                  &               pmld(:,:), pmld(:,:), pinc_chltot_2d(:,:), cchl_p_2d(:,:), &
                  &               nbal_active, phyt_avg_bkg_2d(:,:),                         &
                  &               gl_active, pgrow_avg_bkg_2d(:,:), ploss_avg_bkg_2d(:,:),   &
                  &               subsurf_active, deepneg_active,                            &
                  &               deeppos_active, nutprof_active,                            &
                  &               bstate_2d, outincs_2d,                                     &
                  &               diag_active, diag,                                         &
                  &               diag_fulldepth_active, diag_fulldepth_2d )

               outincs_3d(:,:,jk,:) = outincs_2d(:,:,1,:)
            END DO
         ENDIF
         
         ! Loop over each grid point partioning the increments
         phyto_balinc(:,:,:,:) = 0.0
         DO jk = 1, jpk
            IF (kdeps == 1) THEN
               jkinc = 1
            ELSE
               IF (jk > kdeps) THEN
                  EXIT
               ENDIF
               jkinc = jk
            ENDIF
            DO jj = 1, jpj
               DO ji = 1, jpi

                  ! Phytoplankton
                  IF ( ( tracer_bkg(ji,jj,jk,jpphn) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jpphd) > 0.0 ) .AND. &
                     & ( pinc_chltot_3d(ji,jj,jkinc) /= 0.0 ) ) THEN
                     IF ( ld_chltot ) THEN
                        ! Phytoplankton nitrogen split up based on existing ratios
                        zfrac_phn = tracer_bkg(ji,jj,jk,jpphn) / &
                           &        (tracer_bkg(ji,jj,jk,jpphn) + tracer_bkg(ji,jj,jk,jpphd))
                        zfrac_phd = tracer_bkg(ji,jj,jk,jpphd) / &
                           &        (tracer_bkg(ji,jj,jk,jpphn) + tracer_bkg(ji,jj,jk,jpphd))
                     ELSE IF ( ld_chldia .AND. ld_chlnon ) THEN
                        ! Phytoplankton nitrogen split up based on assimilation increments
                        zfrac_phn = pinc_chlnon_3d(ji,jj,jkinc) / pinc_chltot_3d(ji,jj,jkinc)
                        zfrac_phd = pinc_chldia_3d(ji,jj,jkinc) / pinc_chltot_3d(ji,jj,jkinc)
                     ENDIF

                     ! Phytoplankton silicate split up based on existing ratios
                     zrat_pds_phd = tracer_bkg(ji,jj,jk,jppds) / tracer_bkg(ji,jj,jk,jpphd)
                     
                     ! Chlorophyll split up based on existing ratios to phytoplankton nitrogen
                     ! Not using pinc_chltot directly as it's only 2D
                     ! This method should give same results at surface as splitting pinc_chltot would
                     zrat_chn_phn = tracer_bkg(ji,jj,jk,jpchn) / tracer_bkg(ji,jj,jk,jpphn)
                     zrat_chd_phd = tracer_bkg(ji,jj,jk,jpchd) / tracer_bkg(ji,jj,jk,jpphd)
                     
                     phyto_balinc(ji,jj,jk,jpphn) = outincs_3d(ji,jj,jk,i_tracer(2)) * zfrac_phn
                     phyto_balinc(ji,jj,jk,jpphd) = outincs_3d(ji,jj,jk,i_tracer(2)) * zfrac_phd
                     phyto_balinc(ji,jj,jk,jppds) = phyto_balinc(ji,jj,jk,jpphd) * zrat_pds_phd
                     phyto_balinc(ji,jj,jk,jpchn) = phyto_balinc(ji,jj,jk,jpphn) * zrat_chn_phn
                     phyto_balinc(ji,jj,jk,jpchd) = phyto_balinc(ji,jj,jk,jpphd) * zrat_chd_phd
                  ENDIF

                  ! Zooplankton nitrogen split up based on existing ratios
                  IF ( ( tracer_bkg(ji,jj,jk,jpzmi) > 0.0 ) .AND. ( tracer_bkg(ji,jj,jk,jpzme) > 0.0 ) ) THEN
                     zfrac_zmi = tracer_bkg(ji,jj,jk,jpzmi) / &
                        &        (tracer_bkg(ji,jj,jk,jpzmi) + tracer_bkg(ji,jj,jk,jpzme))
                     zfrac_zme = tracer_bkg(ji,jj,jk,jpzme) / &
                        &        (tracer_bkg(ji,jj,jk,jpzmi) + tracer_bkg(ji,jj,jk,jpzme))
                     phyto_balinc(ji,jj,jk,jpzmi) = outincs_3d(ji,jj,jk,i_tracer(3)) * zfrac_zmi
                     phyto_balinc(ji,jj,jk,jpzme) = outincs_3d(ji,jj,jk,i_tracer(3)) * zfrac_zme
                  ENDIF

                  ! Nitrogen nutrient straight from balancing scheme
                  phyto_balinc(ji,jj,jk,jpdin) = outincs_3d(ji,jj,jk,i_tracer(1))

                  ! Nitrogen detritus straight from balancing scheme
                  phyto_balinc(ji,jj,jk,jpdet) = outincs_3d(ji,jj,jk,i_tracer(4))

                  ! DIC straight from balancing scheme
                  phyto_balinc(ji,jj,jk,jpdic) = outincs_3d(ji,jj,jk,i_tracer(5))

                  ! Alkalinity straight from balancing scheme
                  phyto_balinc(ji,jj,jk,jpalk) = outincs_3d(ji,jj,jk,i_tracer(6))

                  ! Remove diatom silicate increment from nutrient silicate to conserve mass
                  IF ( ( tracer_bkg(ji,jj,jk,jpsil) - phyto_balinc(ji,jj,jk,jppds) ) > 0.0 ) THEN
                     phyto_balinc(ji,jj,jk,jpsil) = phyto_balinc(ji,jj,jk,jppds) * (-1.0)
                  ENDIF

                  ! Carbon detritus based on existing ratios
                  IF ( ( tracer_bkg(ji,jj,jk,jpdet) > 0.0 ) .AND. ( tracer_bkg(ji,jj,jk,jpdtc) > 0.0 ) ) THEN
                     zrat_dtc_det = tracer_bkg(ji,jj,jk,jpdtc) / tracer_bkg(ji,jj,jk,jpdet)
                     phyto_balinc(ji,jj,jk,jpdtc) = phyto_balinc(ji,jj,jk,jpdet) * zrat_dtc_det
                  ENDIF

                  ! Do nothing with iron or oxygen for the time being
                  phyto_balinc(ji,jj,jk,jpfer) = 0.0
                  phyto_balinc(ji,jj,jk,jpoxy) = 0.0
                  
               END DO
            END DO
         END DO
      
      ELSE   ! No nitrogen balancing
      
         ! Initialise individual chlorophyll increments to zero
         phyto_balinc(:,:,:,jpchn) = 0.0
         phyto_balinc(:,:,:,jpchd) = 0.0
         
         ! Split up total surface chlorophyll increments
         DO jk = 1, kdeps
            DO jj = 1, jpj
               DO ji = 1, jpi
                  IF ( ( tracer_bkg(ji,jj,jk,jpchn) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jpchd) > 0.0 ) ) THEN
                     IF ( ld_chltot ) THEN
                        ! Chlorophyll split up based on existing ratios
                        zfrac_chn = tracer_bkg(ji,jj,jk,jpchn) / &
                           &        ( tracer_bkg(ji,jj,jk,jpchn) + tracer_bkg(ji,jj,jk,jpchd) )
                        zfrac_chd = tracer_bkg(ji,jj,jk,jpchd) / &
                           &        ( tracer_bkg(ji,jj,jk,jpchn) + tracer_bkg(ji,jj,jk,jpchd) )
                        phyto_balinc(ji,jj,jk,jpchn) = pinc_chltot_3d(ji,jj,jk) * zfrac_chn
                        phyto_balinc(ji,jj,jk,jpchd) = pinc_chltot_3d(ji,jj,jk) * zfrac_chd
                     ENDIF
                     IF( ld_chldia ) THEN
                        phyto_balinc(ji,jj,jk,jpchd) = pinc_chldia_3d(ji,jj,jk)
                     ENDIF
                     IF( ld_chlnon ) THEN
                        phyto_balinc(ji,jj,jk,jpchn) = pinc_chlnon_3d(ji,jj,jk)
                     ENDIF

                     ! Maintain stoichiometric ratios of nitrogen and silicate
                     IF ( ld_chltot .OR. ld_chlnon ) THEN
                        zrat_phn_chn = tracer_bkg(ji,jj,jk,jpphn) / tracer_bkg(ji,jj,jk,jpchn)
                        phyto_balinc(ji,jj,jk,jpphn) = phyto_balinc(ji,jj,jk,jpchn) * zrat_phn_chn
                     ENDIF
                     IF ( ld_chltot .OR. ld_chldia ) THEN
                        zrat_phd_chd = tracer_bkg(ji,jj,jk,jpphd) / tracer_bkg(ji,jj,jk,jpchd)
                        phyto_balinc(ji,jj,jk,jpphd) = phyto_balinc(ji,jj,jk,jpchd) * zrat_phd_chd
                        zrat_pds_chd = tracer_bkg(ji,jj,jk,jppds) / tracer_bkg(ji,jj,jk,jpchd)
                        phyto_balinc(ji,jj,jk,jppds) = phyto_balinc(ji,jj,jk,jpchd) * zrat_pds_chd
                     ENDIF
                  ENDIF
               END DO
            END DO
         END DO
         
         IF (kdeps == 1) THEN
            ! Propagate through mixed layer
            DO jj = 1, jpj
               DO ji = 1, jpi
                  !
                  jkmax = jpk-1
                  DO jk = jpk-1, 1, -1
                     IF ( ( pmld(ji,jj) >  gdepw_n(ji,jj,jk)   ) .AND. &
                        & ( pmld(ji,jj) <= gdepw_n(ji,jj,jk+1) ) ) THEN
                        pmld(ji,jj) = gdepw_n(ji,jj,jk+1)
                        jkmax = jk
                     ENDIF
                  END DO
                  !
                  DO jk = 2, jkmax
                     phyto_balinc(ji,jj,jk,jpchn) = phyto_balinc(ji,jj,1,jpchn)
                     phyto_balinc(ji,jj,jk,jpchd) = phyto_balinc(ji,jj,1,jpchd)
                     phyto_balinc(ji,jj,jk,jpphn) = phyto_balinc(ji,jj,1,jpphn)
                     phyto_balinc(ji,jj,jk,jpphd) = phyto_balinc(ji,jj,1,jpphd)
                     phyto_balinc(ji,jj,jk,jppds) = phyto_balinc(ji,jj,1,jppds)
                  END DO
                  !
               END DO
            END DO
         ENDIF

         ! Set other balancing increments to zero
         phyto_balinc(:,:,:,jpzmi) = 0.0
         phyto_balinc(:,:,:,jpzme) = 0.0
         phyto_balinc(:,:,:,jpdin) = 0.0
         phyto_balinc(:,:,:,jpsil) = 0.0
         phyto_balinc(:,:,:,jpfer) = 0.0
         phyto_balinc(:,:,:,jpdet) = 0.0
         phyto_balinc(:,:,:,jpdtc) = 0.0
         phyto_balinc(:,:,:,jpdic) = 0.0
         phyto_balinc(:,:,:,jpalk) = 0.0
         phyto_balinc(:,:,:,jpoxy) = 0.0

      ENDIF
      
      ! If performing extra tidal mixing in the Indonesian Throughflow,
      ! increments have been found to make the carbon cycle unstable
      ! Therefore, mask these out
      IF ( ln_tmx_itf ) THEN
         DO jn = 1, jptra
            DO jk = 1, jpk
               phyto_balinc(:,:,jk,jn) = phyto_balinc(:,:,jk,jn) * ( 1.0 - mask_itf(:,:) )
            END DO
         END DO
      ENDIF

   END SUBROUTINE asm_phyto_bal_medusa

#else
   !!----------------------------------------------------------------------
   !!   Default option : Empty routine
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE asm_phyto_bal_medusa( kdeps,                          &
      &                             ld_chltot,                      &
      &                             pinc_chltot_3d,                    &
      &                             ld_chldia,                      &
      &                             pinc_chldia_3d,                    &
      &                             ld_chlnon,                      &
      &                             pinc_chlnon_3d,                    &
      &                             ld_phytot,                      &
      &                             pinc_phytot_3d,                    &
      &                             ld_phydia,                      &
      &                             pinc_phydia_3d,                    &
      &                             ld_phynon,                      &
      &                             pinc_phynon_3d,                    &
      &                             pincper,                        &
      &                             p_maxchlinc, ld_phytobal, pmld, &
      &                             pgrow_avg_bkg_3d, ploss_avg_bkg_3d,   &
      &                             phyt_avg_bkg_3d, mld_max_bkg,      &
      &                             tracer_bkg, phyto_balinc )
      INTEGER :: kdeps
      LOGICAL :: ld_chltot
      REAL    :: pinc_chltot_3d(:,:,:)
      LOGICAL :: ld_chldia
      REAL    :: pinc_chldia_3d(:,:,:)
      LOGICAL :: ld_chlnon
      REAL    :: pinc_chlnon_3d(:,:,:)
      LOGICAL :: ld_phytot
      REAL    :: pinc_phytot_3d(:,:,:)
      LOGICAL :: ld_phydia
      REAL    :: pinc_phydia_3d(:,:,:)
      LOGICAL :: ld_phynon
      REAL    :: pinc_phynon_3d(:,:,:)
      REAL    :: pincper
      REAL    :: p_maxchlinc
      LOGICAL :: ld_phytobal
      REAL    :: pmld(:,:)
      REAL    :: pgrow_avg_bkg_3d(:,:,:)
      REAL    :: ploss_avg_bkg_3d(:,:,:)
      REAL    :: phyt_avg_bkg_3d(:,:,:)
      REAL    :: mld_max_bkg(:,:)
      REAL    :: tracer_bkg(:,:,:,:)
      REAL    :: phyto_balinc(:,:,:,:)
      WRITE(*,*) 'asm_phyto_bal_medusa: You should not have seen this print! error?'
   END SUBROUTINE asm_phyto_bal_medusa
#endif

   !!======================================================================
END MODULE asmphytobal_medusa
